from __future__ import annotations

import os
import sys
from importlib import import_module

from django.conf import settings
from django.core.management.color import color_style
from django.utils.translation import gettext as _
from edc_visit_schedule.site_visit_schedules import site_visit_schedules

from .forms_reference import FormsReference

style = color_style()


def make_forms_reference(
    app_label: str = None,
    admin_site_name: str = None,
    visit_schedule_name: str = None,
    title: str = None,
    path: str | None = None,
):
    module = import_module(app_label)
    admin_site = getattr(getattr(module, "admin_site"), admin_site_name)
    visit_schedule = site_visit_schedules.get_visit_schedule(visit_schedule_name)
    title = title or _("%(title_app)s Forms Reference") % dict(
        title_app=app_label.upper()
    )
    sys.stdout.write(
        style.MIGRATE_HEADING(f"Refreshing CRF reference document for {app_label}\n")
    )
    doc_folder = os.path.join(settings.BASE_DIR, "docs")
    if not os.path.exists(doc_folder):
        os.mkdir(doc_folder)

    forms = FormsReference(
        visit_schedules=[visit_schedule],
        admin_site=admin_site,
        title=title,
        add_per_form_timestamp=False,
    )

    path = os.path.join(doc_folder, "forms_reference.md")
    forms.to_file(path=path, overwrite=True)

    print(path)
    print("Done.")



from __future__ import annotations

import os
from datetime import datetime


class MarkdownWriter:
    def __init__(self, path: str | None = None, overwrite: bool | None = None):
        self.path = self.get_path(path=path, overwrite=overwrite)

    @staticmethod
    def get_path(path: str | None = None, overwrite: bool | None = None) -> str:
        if not path:
            timestamp = datetime.today().strftime("%Y%m%d%H%M")
            path = f"forms_{timestamp}.md"
        if os.path.exists(path):
            if overwrite:
                os.remove(path)
            else:
                raise FileExistsError(f"File exists. Got '{path}'")
        return path

    @staticmethod
    def to_markdown(markdown: list[str] = None) -> str:
        """Returns the markdown as a text string."""
        return "\n".join(markdown)

    def to_file(
        self,
        markdown: list[str] = None,
        pad: int | None = None,
        append: bool | None = None,
        prepend: bool | None = None,
    ) -> None:
        markdown = self.to_markdown(markdown=markdown)
        if pad:
            markdown = markdown + ("\n" * pad)
        if append:
            self._append(markdown)
        elif prepend:
            self._prepend(markdown)
        else:
            self._write(markdown)

    def _write(self, markdown: str = None, mode: str | None = None) -> None:
        mode = mode or "w"
        with open(self.path, mode) as f:
            f.write(markdown)

    def _append(self, markdown) -> None:
        mode = "a"
        self._write(markdown=markdown, mode=mode)

    def _prepend(self, markdown=None) -> None:
        mode = "r+"
        with open(self.path, mode) as f:
            content = f.read()
            f.seek(0, 0)
            f.write(markdown + "\n" + content)



import sys
from pathlib import Path

from edc_test_settings.default_test_settings import DefaultTestSettings

app_name = "edc_form_describer"
base_dir = Path(__file__).absolute().parent.parent

project_settings = DefaultTestSettings(
    calling_file=__file__,
    APP_NAME=app_name,
    BASE_DIR=base_dir,
    ETC_DIR=base_dir / "tests" / "etc",
    TEST_DIR=base_dir / "tests",
    EDC_RANDOMIZATION_LIST_PATH=base_dir / "tests" / "etc",
    HOLIDAY_FILE=base_dir / "tests" / "holidays.csv",
    # ROOT_URLCONF="tests.urls",
    DJANGO_REVISION_IGNORE_WORKING_DIR=True,
    DJANGO_CRYPTO_FIELDS_KEY_PATH=base_dir / "tests" / "etc",
    SILENCED_SYSTEM_CHECKS=[
        "sites.E101",
        "edc_navbar.E002",
        "edc_navbar.E003",
        "edc_consent.E001",
        "edc_sites.E001",
        "edc_sites.E002",
    ],
    EDC_AUTH_SKIP_SITE_AUTHS=True,
    EDC_AUTH_SKIP_AUTH_UPDATER=True,
    INSTALLED_APPS=[
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.staticfiles",
        "django.contrib.sites",
        "django_crypto_fields.apps.AppConfig",
        "django_revision.apps.AppConfig",
        "simple_history",
        "edc_sites.apps.AppConfig",
        "edc_action_item.apps.AppConfig",
        "edc_appointment.apps.AppConfig",
        "edc_fieldsets.apps.AppConfig",
        "edc_notification.apps.AppConfig",
        "edc_visit_schedule.apps.AppConfig",
        "edc_visit_tracking.apps.AppConfig",
        "edc_form_describer.apps.AppConfig",
    ],
    add_dashboard_middleware=True,
    use_test_urls=True,
).settings

for k, v in project_settings.items():
    setattr(sys.modules[__name__], k, v)



from __future__ import annotations

import sys
from importlib import import_module
from importlib.metadata import version
from pathlib import Path

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.core.management.color import color_style
from django.utils.translation import gettext as _
from edc_visit_schedule.site_visit_schedules import site_visit_schedules

from edc_form_describer.forms_reference import FormsReference

style = color_style()


def update_forms_reference(
    app_label: str = None,
    admin_site_name: str = None,
    visit_schedule_name: str = None,
    title: str = None,
    filename: str | None = None,
    doc_folder: str | None = None,
):
    module = import_module(app_label)
    default_doc_folder = Path(settings.BASE_DIR / "docs")
    filename = filename or "forms_reference.md"
    admin_site = getattr(getattr(module, "admin_site"), admin_site_name)
    visit_schedule = site_visit_schedules.get_visit_schedule(visit_schedule_name)
    title = title or _("%(title_app)s Forms Reference") % dict(
        title_app=app_label.upper()
    )
    sys.stdout.write(
        style.MIGRATE_HEADING(f"Refreshing CRF reference document for {app_label}\n")
    )
    doc_folder = doc_folder or default_doc_folder
    if doc_folder == default_doc_folder and not default_doc_folder.exists():
        doc_folder.mkdir(parents=False, exist_ok=False)

    forms = FormsReference(
        visit_schedules=[visit_schedule],
        admin_site=admin_site,
        title=f"{title} v{version(settings.APP_NAME)}",
        add_per_form_timestamp=False,
    )

    path = doc_folder / filename
    forms.to_file(path=path, overwrite=True)

    print(path)
    print("Done.")


class Command(BaseCommand):
    help = "Update forms reference document (.md)"

    def add_arguments(self, parser):
        parser.add_argument(
            "--app-label",
            dest="app_label",
            default=None,
        )

        parser.add_argument(
            "--admin-site",
            dest="admin_site_name",
            default=None,
        )

        parser.add_argument(
            "--visit-schedule",
            dest="visit_schedule_name",
            default=None,
        )

        parser.add_argument(
            "--title",
            dest="title",
            default=None,
        )

        parser.add_argument(
            "--doc_folder",
            dest="doc_folder",
            default=None,
        )

    def handle(self, *args, **options):
        app_label = options["app_label"]
        admin_site_name = options["admin_site_name"]
        visit_schedule_name = options["visit_schedule_name"]
        title = options["title"]
        doc_folder = options["doc_folder"]

        if not app_label or not admin_site_name or not visit_schedule_name:
            raise CommandError(f"parameter missing. got {options}")

        update_forms_reference(
            app_label=app_label,
            admin_site_name=admin_site_name,
            visit_schedule_name=visit_schedule_name,
            title=title,
            doc_folder=doc_folder,
        )



from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_model_admin.mixins import ModelAdminFormInstructionsMixin

from .forms import MyForm
from .models import MyModel


@admin.register(MyModel)
class MyModelAdmin(ModelAdminFormInstructionsMixin, admin.ModelAdmin):
    form = MyForm

    fieldsets = (
        (
            "Not special fields",
            {"fields": ("subject_visit", "report_datetime", "f1", "f2", "f3")},
        ),
        (
            "Visit Two Additional Questions",
            {"fields": ("f4", "f5")},
        ),
        (
            "Summary",
            {"fields": ("summary_one", "summary_two")},
        ),
        audit_fieldset_tuple,
    )



from tempfile import mkstemp

from django.test import TestCase

from edc_form_describer.form_describer import FormDescriber

from ..admin import MyModelAdmin
from ..models import MyModel


class TestForDescribter(TestCase):
    @staticmethod
    def get_fields_from_fieldset(admin_cls) -> list[str]:
        fields = []
        for _, fields_dict in admin_cls.fieldsets:
            for f in fields_dict["fields"]:
                fields.append(f)
        return fields

    def test_ok(self):
        describer = FormDescriber(admin_cls=MyModelAdmin, include_hidden_fields=True)
        txt = " ".join(describer.markdown)
        fields = self.get_fields_from_fieldset(MyModelAdmin)
        for f in MyModel._meta.get_fields():
            if f.name in fields:
                self.assertIn(str(f.verbose_name), txt)

    def test_to_file(self):
        tmp, name = mkstemp()
        describer = FormDescriber(admin_cls=MyModelAdmin, include_hidden_fields=True)
        describer.to_file(path=name, overwrite=True)
        with open(name, "r") as describer_file:
            txt = describer_file.read()
            fields = self.get_fields_from_fieldset(MyModelAdmin)
            for f in MyModel._meta.get_fields():
                if f.name in fields:
                    self.assertIn(str(f.verbose_name), txt)
